import React, { useState, useEffect } from 'react';
import axios from 'axios';
import { Route, Redirect, RouteProps } from 'react-router-dom';
import { NonAuthRoutes } from 'components/api/routes';

/**
 * A wrapper for <Route> that redirects to the login screen if you're not yet authenticated.
 * */

type Props = {
  Component: React.FC<RouteProps>;
  path: string;
  requiredRoles: string[];
};

/* eslint-disable react/jsx-props-no-spreading */
export const PrivateRoute = ({
  Component,
  path,
  requiredRoles,
}: Props): JSX.Element => {
  const [auth, setAuth] = useState<boolean>(false);
  const [loading, setLoading] = useState<boolean>(false);

  useEffect(() => {
    const fetch = async (): Promise<unknown> => {
      const result = await axios('/api/web/login/is_authenticated');
      setAuth(result.data.is_authenticated);
      setLoading(true);
      return null;
    };
    fetch();
  }, []);

  const currentRole = String(sessionStorage.getItem('ROLE'));
  const userHasRequiredRole = requiredRoles.includes(currentRole);
  const message = userHasRequiredRole
    ? 'Please log in to view this page'
    : 'Your role is not allowed';

  return !loading ? (
    <p>loading</p>
  ) : (
    <Route
      exact={false}
      path={path}
      render={(props: RouteProps) =>
        auth && userHasRequiredRole ? (
          <Component {...props} />
        ) : (
          <Redirect
            to={{
              pathname: userHasRequiredRole
                ? NonAuthRoutes.signIn
                : NonAuthRoutes.unauthorized,
              state: {
                message,
                requestedPath: path,
              },
            }}
          />
        )
      }
    />
  );
};
